@@ -1050,7 +1050,7 @@ static void stress_iomix_readahead(
 }
 #endif
 
-static stress_iomix_func iomix_funcs[] = {
+static const stress_iomix_func iomix_funcs[] = {
 	stress_iomix_wr_seq_bursts,
 	stress_iomix_wr_rnd_bursts,
 	stress_iomix_wr_seq_slow,
@@ -1099,7 +1099,7 @@ static int stress_iomix(stress_args_t *args)
 {
 	int fd, ret;
 	char filename[PATH_MAX];
-	off_t iomix_bytes = DEFAULT_IOMIX_BYTES;
+	off_t iomix_bytes, iomix_bytes_total = DEFAULT_IOMIX_BYTES;
 	const size_t page_size = args->page_size;
 	size_t i;
 	stress_pid_t *s_pids, *s_pids_head = NULL;
@@ -1112,7 +1112,8 @@ static int stress_iomix(stress_args_t *args)
 
 	s_pids = stress_sync_s_pids_mmap(MAX_IOMIX_PROCS);
 	if (s_pids == MAP_FAILED) {
-		pr_inf_skip("%s: failed to mmap %zu PIDs, skipping stressor\n", args->name, MAX_IOMIX_PROCS);
+		pr_inf_skip("%s: failed to mmap %zu PIDs%s, skipping stressor\n",
+			args->name, MAX_IOMIX_PROCS, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -1127,17 +1128,23 @@ static int stress_iomix(stress_args_t *args)
 		goto tidy_s_pids;
 	}
 
-	if (!stress_get_setting("iomix-bytes", &iomix_bytes)) {
+	if (!stress_get_setting("iomix-bytes", &iomix_bytes_total)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			iomix_bytes = MAXIMIZED_FILE_SIZE;
+			iomix_bytes_total = MAXIMIZED_FILE_SIZE;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			iomix_bytes = MIN_IOMIX_BYTES;
+			iomix_bytes_total = MIN_IOMIX_BYTES;
 	}
-	iomix_bytes /= args->instances;
-	if (iomix_bytes < (off_t)MIN_IOMIX_BYTES)
+	iomix_bytes = iomix_bytes_total / args->instances;
+	if (iomix_bytes < (off_t)MIN_IOMIX_BYTES) {
 		iomix_bytes = (off_t)MIN_IOMIX_BYTES;
-	if (iomix_bytes < (off_t)page_size)
+		iomix_bytes_total = iomix_bytes * args->instance;
+	}
+	if (iomix_bytes < (off_t)page_size) {
 		iomix_bytes = (off_t)page_size;
+		iomix_bytes_total = iomix_bytes * args->instance;
+	}
+	if (stress_instance_zero(args))
+		stress_fs_usage_bytes(args, iomix_bytes, iomix_bytes_total);
 
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0) {
@@ -1242,7 +1249,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_iomix_info = {
 	.stressor = stress_iomix,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
