@@ -67,7 +67,7 @@ static int stress_l1cache_info_check(
 	const uint64_t sz = size ? size :
 		(uint64_t)ways * (uint64_t)sets * (uint64_t)line_size;
 
-	if (args->instance == 0) {
+	if (stress_instance_zero(args)) {
 		char szstr[64];
 
 		stress_uint64_to_str(szstr, sizeof(szstr), sz, 0, true);
@@ -528,8 +528,10 @@ static int stress_l1cache(stress_args_t *args)
 				PROT_READ | PROT_WRITE,
 				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (cache == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap cache test buffer, skipping stressor, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_inf_skip("%s: cannot mmap cache %" PRIu32" byte test buffer%s, "
+			"skipping stressor, errno=%d (%s)\n",
+			args->name, l1cache_size << 2,
+			stress_get_memfree_str(), errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(cache, l1cache_size, "l1cache");
@@ -576,7 +578,7 @@ static int stress_l1cache(stress_args_t *args)
 
 const stressor_info_t stress_l1cache_info = {
 	.stressor = stress_l1cache,
-	.class = CLASS_CPU_CACHE,
+	.classifier = CLASS_CPU_CACHE,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
