@@ -23,6 +23,12 @@
 
 #include <sys/ioctl.h>
 
+#define MIN_PIPE_SIZE		(4096)
+#define MAX_PIPE_SIZE		(1024 * 1024)
+
+#define MIN_PIPE_DATA_SIZE	(8)
+#define MAX_PIPE_DATA_SIZE	(4096)
+
 static const stress_help_t help[] = {
 	{ "p N", "pipe N",		"start N workers exercising pipe I/O" },
 	{ NULL,	"pipe-data-size N",	"set pipe size of each pipe write to N bytes" },
@@ -537,7 +543,12 @@ static int stress_pipe(stress_args_t *args)
 	bool pipe_vmsplice = false;
 
 	(void)stress_get_setting("pipe-vmsplice", &pipe_vmsplice);
-	(void)stress_get_setting("pipe-data-size", &pipe_data_size);
+	if (!stress_get_setting("pipe-data-size", &pipe_data_size)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			pipe_data_size = MAX_PIPE_DATA_SIZE;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			pipe_data_size = MIN_PIPE_DATA_SIZE;
+	}
 
 	if (stress_sig_stop_stressing(args->name, SIGPIPE) < 0)
 		return EXIT_FAILURE;
@@ -563,7 +574,7 @@ static int stress_pipe(stress_args_t *args)
 	}
 	if (pipe_vmsplice) {
 		pipe_vmsplice = false;
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: no pipe packet mode with O_DIRECT, disabled pipe vmsplicing\n", args->name);
 	}
 #endif
@@ -572,7 +583,12 @@ static int stress_pipe(stress_args_t *args)
 	{
 		size_t pipe_size = 0;
 
-		(void)stress_get_setting("pipe-size", &pipe_size);
+		if (!stress_get_setting("pipe-size", &pipe_size)) {
+			if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+				pipe_size = MAX_PIPE_SIZE;
+			if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+				pipe_size = MIN_PIPE_SIZE;
+		}
 		if (pipe_size > 0) {
 			pipe_change_size(args, pipefds[0], pipe_size);
 			pipe_change_size(args, pipefds[1], pipe_size);
@@ -584,7 +600,7 @@ static int stress_pipe(stress_args_t *args)
 	pipe_rd_size = pipe_get_size(pipefds[0]);
 	pipe_wr_size = pipe_get_size(pipefds[1]);
 
-	if (args->instance == 0) {
+	if (stress_instance_zero(args)) {
 		pr_dbg("%s: pipe read size %zuK, pipe write size %zuK\n",
 			args->name, pipe_rd_size >> 10, pipe_wr_size >> 10);
 	}
@@ -595,8 +611,10 @@ static int stress_pipe(stress_args_t *args)
 				PROT_READ | PROT_WRITE,
 				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (buf_rd == MAP_FAILED) {
-		pr_inf_skip("%s: failed to mmap %zu byte read buffer, skipping stressor\n",
-			args->name, buf_rd_size);
+		pr_inf_skip("%s: failed to mmap %zu byte read buffer%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, buf_rd_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		(void)close(pipefds[0]);
 		(void)close(pipefds[1]);
 		return EXIT_NO_RESOURCE;
@@ -609,8 +627,10 @@ static int stress_pipe(stress_args_t *args)
 				PROT_READ | PROT_WRITE,
 				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (buf_wr == MAP_FAILED) {
-		pr_inf_skip("%s: failed to mmap %zu byte write buffer, skipping stressor\n",
-			args->name, buf_wr_size);
+		pr_inf_skip("%s: failed to mmap %zu byte write buffer%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, buf_wr_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		(void)munmap((void *)buf_rd, buf_rd_size);
 		(void)close(pipefds[0]);
 		(void)close(pipefds[1]);
@@ -632,7 +652,7 @@ again:
 		(void)close(pipefds[1]);
 		(void)munmap((void *)buf_wr, buf_wr_size);
 		(void)munmap((void *)buf_rd, buf_rd_size);
-		if (!stress_continue(args))
+		if (UNLIKELY(!stress_continue(args)))
 			goto finish;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -709,17 +729,17 @@ finish:
 
 static const stress_opt_t opts[] = {
 #if defined(F_SETPIPE_SZ)
-	{ OPT_pipe_size,      "pipe-size",      TYPE_ID_SIZE_T_BYTES_VM, 4096, 1024 * 1024, NULL },
+	{ OPT_pipe_size,      "pipe-size",      TYPE_ID_SIZE_T_BYTES_VM, MIN_PIPE_SIZE, MAX_PIPE_SIZE, NULL },
 #endif
 	/* FIXME: was min = 8, max = stress_get_page_size() */
-	{ OPT_pipe_data_size, "pipe-data-size", TYPE_ID_SIZE_T_BYTES_VM, 8, 4096, NULL },
+	{ OPT_pipe_data_size, "pipe-data-size", TYPE_ID_SIZE_T_BYTES_VM, MIN_PIPE_DATA_SIZE, MAX_PIPE_DATA_SIZE, NULL },
 	{ OPT_pipe_vmsplice,  "pipe-vmsplice",  TYPE_ID_BOOL, 0, 1, NULL },
 	END_OPT,
 };
 
 const stressor_info_t stress_pipe_info = {
 	.stressor = stress_pipe,
-	.class = CLASS_PIPE_IO | CLASS_MEMORY | CLASS_OS | CLASS_IPC,
+	.classifier = CLASS_PIPE_IO | CLASS_MEMORY | CLASS_OS | CLASS_IPC,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
