@@ -367,7 +367,7 @@ retry:
 		fd = socket(sctp_domain, SOCK_STREAM, IPPROTO_SCTP);
 		if (UNLIKELY(fd < 0)) {
 			if (errno == EPROTONOSUPPORT) {
-				if (args->instance == 0)
+				if (stress_instance_zero(args))
 					pr_inf_skip("%s: SCTP protocol not supported, skipping stressor\n",
 						args->name);
 				return EXIT_NO_RESOURCE;
@@ -488,7 +488,7 @@ static int OPTIMIZE3 stress_sctp_server(
 	fd = socket(sctp_domain, SOCK_STREAM, IPPROTO_SCTP);
 	if (fd < 0) {
 		if (errno == EPROTONOSUPPORT) {
-			if (args->instance == 0)
+			if (stress_instance_zero(args))
 				pr_inf_skip("%s: SCTP protocol not supported, skipping stressor\n",
 					args->name);
 			rc = EXIT_NO_RESOURCE;
@@ -702,7 +702,7 @@ finish:
 
 const stressor_info_t stress_sctp_info = {
 	.stressor = stress_sctp,
-	.class = CLASS_NETWORK,
+	.classifier = CLASS_NETWORK,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -710,7 +710,7 @@ const stressor_info_t stress_sctp_info = {
 #else
 const stressor_info_t stress_sctp_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_NETWORK,
+	.classifier = CLASS_NETWORK,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
