@@ -1457,7 +1457,7 @@ static int stress_zlib_inflate(
 	stream_inf.avail_in = 0;
 	stream_inf.next_in = Z_NULL;
 
-	if (args->instance == 0) {
+	if (stress_instance_zero(args)) {
 		pr_dbg("INF: lvl=%d mem-lvl=%d wbits=%d strategy=%d\n",
 			zlib_args.level, zlib_args.mem_level, zlib_args.window_bits,
 			zlib_args.strategy);
@@ -1597,7 +1597,7 @@ static int stress_zlib_deflate(
 	if (zlib_args.window_bits > 31)
 		zlib_args.window_bits = zlib_args.window_bits - 32;
 
-	if (args->instance == 0) {
+	if (stress_instance_zero(args)) {
 		pr_dbg("DEF: lvl=%d mem-lvl=%d wbits=%d strategy=%d stream-bytes=%llu\n",
 			zlib_args.level, zlib_args.mem_level, zlib_args.window_bits,
 			zlib_args.strategy, (unsigned long long int)zlib_args.stream_bytes);
@@ -1767,8 +1767,10 @@ static int stress_zlib(stress_args_t *args)
 			PROT_READ | PROT_WRITE,
 			MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (shared_checksums == MAP_FAILED) {
-		pr_inf("%s: failed to mmap %zd bytes, skipping stressor\n",
-			args->name, sizeof(*shared_checksums));
+		pr_inf("%s: failed to mmap %zu bytes%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, sizeof(*shared_checksums),
+			stress_get_memfree_str(), errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
 	stress_set_vma_anon_name(shared_checksums, sizeof(*shared_checksums), "zlib-checksums");
@@ -1869,7 +1871,7 @@ again:
 
 const stressor_info_t stress_zlib_info = {
 	.stressor = stress_zlib,
-	.class = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY | CLASS_COMPUTE,
+	.classifier = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY | CLASS_COMPUTE,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
@@ -1877,7 +1879,7 @@ const stressor_info_t stress_zlib_info = {
 #else
 const stressor_info_t stress_zlib_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY | CLASS_COMPUTE,
+	.classifier = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY | CLASS_COMPUTE,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built without zlib library support"
